#property strict

#include "Indicator.mqh"
#include "BufferScale.mqh"
#include "..\Quotes_FXOPEN.mqh"

#define PAUSE_CYCLE 150

class BOTHPRICES : public INDICATOR
{
private:
  datetime StartDate; //  
  double MarkUp;      //  ( + )   
  bool ShowVol;       //  " " (HighAsk - LowBid)
  int Pause;

  datetime CalcIndicator( const datetime FirstDate )
  {
    QUOTES_FXOPEN Quotes(::Symbol(), FirstDate);

    Quotes.SetMarkUp(this.MarkUp);

    const int Amount = Quotes.GetAmount();
    const double point = ::SymbolInfoDouble(Quotes.GetSymbolName(), SYMBOL_POINT);

    int FirstIndex = 0;      //     -
    int LastIndex = INT_MAX; //     -

    for (int i = 0; i < Amount; i++)
    {
    // QUOTES    ,    MT4
      const int j = ::iBarShift(::Symbol(), ::Period(), Quotes[i].time - ::PeriodSeconds(Quotes.GetTimeFrame()), TRUE);

      if (j != -1)
      {
        if (this.ShowVol)
        {
          this.Buffers[0].Buffer[j] = Quotes[i].ask.max * point;
          this.Buffers[1].Buffer[j] = Quotes[i].bid.min * point;
        }

        this.Buffers[2].Buffer[j] = Quotes[i].bid.max * point;
        this.Buffers[3].Buffer[j] = Quotes[i].ask.min * point;
      }

      if (FirstIndex < j)
        FirstIndex = j;

      if (LastIndex > j)
        LastIndex = j;
    }

    if (FirstIndex >= LastIndex)
    {
      FirstIndex -= LastIndex - 1;

      //  
      ::ArrayCopy(this.Buffers[4].Buffer, this.Buffers[2].Buffer, LastIndex, LastIndex, FirstIndex);
      ::ArrayCopy(this.Buffers[5].Buffer, this.Buffers[3].Buffer, LastIndex, LastIndex, FirstIndex);

      if (!this.ShowVol)
      {
        this.Buffers[0].SetEmptyValue(LastIndex, FirstIndex);
        this.Buffers[1].SetEmptyValue(LastIndex, FirstIndex);
      }
    }

    return(Quotes.GetLastTime() - PeriodSeconds(Quotes.GetTimeFrame()));
  }

  // , Freq -  
  static void ChangeBufferColor( BUFFER* &Buffer, const double Freq = 1, const int PartColor = 0xFF, const bool Redraw = TRUE )
  {
    static int Add = 1;

    const int Tmp = (int)(0xFF * EVENTBASE::GetTimerInterval() * Freq / 1000);

    Add = (Add > 0) ? Tmp : -Tmp;

    color Color = Buffer.GetColor();
    uchar Byte = (uchar)((Color & PartColor) / (PartColor / 0xFF));

    if (((Add < 0) && (Byte < -Add)) || ((Add > 0) && (Byte > 0xFF - Add)))
    {
      Add = - Add;

      Byte = (uchar)((Add > 0) ? Add: 0xFF + Add);
    }
    else
      Byte = (uchar)(Byte + Add);

    Color = (color)((Color & (0xFFFFFF - PartColor)) + Byte * PartColor / 0xFF);

    Buffer.SetColor(Color, Redraw);

    return;
  }

// -  
  virtual void EventTimer( void )
  {
    if (this.Buffers[4].GetStyle() != STYLE_DASH)
    {
      this.Buffers[4].SetStyle(STYLE_DASH, FALSE);
      this.Buffers[5].SetStyle(STYLE_DASHDOT, FALSE);
    }
    else
    {
      this.Buffers[4].SetStyle(STYLE_DOT, FALSE);
      this.Buffers[5].SetStyle(STYLE_DASHDOTDOT, FALSE);
    }

//    this.ChangeBufferColor(this.Buffers[2], 1, 0xFF00, FALSE);
    this.ChangeBufferColor(this.Buffers[3]);

    return;
  }

public:
  void Init( const datetime tStartDate = NULL, const double dMarkUp = 0, const bool bShowVol = FALSE, const double dPause = 0 )
  {
    this.StartDate = tStartDate;
    this.MarkUp = dMarkUp;
    this.ShowVol = bShowVol;
    this.Pause = (int)dPause;

    return;
  }

  BOTHPRICES( const datetime tStartDate = NULL, const double dMarkUp = 0, const bool bShowVol = FALSE, const double dPause = 0,
              const bool AddEvent = FALSE  ) : INDICATOR(AddEvent)
  {
    this.Init(tStartDate, dMarkUp, bShowVol, dPause);

    const string Names[] = {"HighAsk", "LowBid", "HighBid", "LowAsk", "HighBid_Line", "LowAsk_Line"};
    const int Types[] = {DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_LINE, DRAW_LINE};
    const color Colors[] = {clrYellow, clrYellow, clrBlue, clrRed, clrDeepSkyBlue, clrRed};

    const int Size = ::ArraySize(Names);

    ::ArrayResize(this.Buffers, Size);

    for (int i = 0; i < Size; i++)
      if ((i == 2) || (i == 3))
        this.Buffers[i] = new BUFFERSCALE(Types[i], EMPTY, EMPTY, Colors[i], Names[i], TRUE, TRUE); //   
      else
        this.Buffers[i] = new BUFFER(Types[i], EMPTY, EMPTY, Colors[i], Names[i], TRUE, TRUE);

    ::IndicatorSetInteger(INDICATOR_DIGITS, ::_Digits);

    ::EventSetMillisecondTimer(PAUSE_CYCLE);
  }

  ~BOTHPRICES( void )
  {
    ::EventKillTimer();
  }

  virtual int Calculate( const int rates_total,
                         const int prev_calculated,
                         const datetime &time[],
                         const double &open[],
                         const double &high[],
                         const double &low[],
                         const double &close[],
                         const long &tick_volume[],
                         const long &volume[],
                         const int &spread[] )
  {
    static datetime LastTime = NULL;

    const datetime FirstTime = time[this.CorrectIndex(rates_total - prev_calculated, rates_total)];

    if (LastTime > FirstTime)
      LastTime = FirstTime;

    if (LastTime < this.StartDate)
      LastTime = this.StartDate;

    // MT4    ,    
    if (prev_calculated == 0)
      this.SetEmpty();

    LastTime = CalcIndicator(LastTime);

    //  ,    
    if (prev_calculated == 0)
      this.ToVisual(this.Pause);
    else
      this.ToVisual();

    return(rates_total);
  }
};