#property strict

#include "Indicator.mqh"
#include "..\ZigZag.mqh"
#include "..\Quotes_FXOPEN.mqh"

class ZIGZAG_BOTHPRICES : public INDICATOR
{
private:
  datetime StartDate; //  
  double MarkUp;      //  ( + ) 
  int MinPips;

  datetime CalcIndicator( const datetime FirstDate )
  {
    QUOTES_FXOPEN Quotes(::Symbol(), this.StartDate);

    Quotes.SetMarkUp(this.MarkUp);

    const ZIGZAG Zz(::GetPointer(Quotes), this.MinPips);

    const int Amount = Zz.GetAmount();
    const double point = ::SymbolInfoDouble(Quotes.GetSymbolName(), SYMBOL_POINT);

    for (int i = 0; i < Amount; i++)
    {
    // QUOTES    ,    MT4
      const int j = ::iBarShift(::Symbol(), ::Period(), Zz[i].time - ::PeriodSeconds(Quotes.GetTimeFrame()), TRUE);

      if (j != -1)
      {
        if ((i & 0x01) == 0)
          this.Buffers[0].Buffer[j] = Zz[i].Price * point;
        else
          this.Buffers[1].Buffer[j] = Zz[i].Price * point;
      }
    }

    return(Quotes.GetLastTime() - ::PeriodSeconds(Quotes.GetTimeFrame()));
  }
public:
  void Init( const datetime tStartDate = NULL, const double dMarkUp = 0, const int iMinPips = 0 )
  {
    this.StartDate = tStartDate;
    this.MarkUp = dMarkUp;
    this.MinPips = iMinPips;

    return;
  }

  ZIGZAG_BOTHPRICES( const datetime tStartDate = NULL, const double dMarkUp = 0, const int iMinPips = 0,
                     const bool AddEvent = FALSE  ) : INDICATOR(AddEvent)
  {
    this.Init(tStartDate, dMarkUp, iMinPips);

    const string Names[] = {"ZigZag_BothPrices_High", "ZigZag_BothPrices_Low"};
    const int Types[] = {DRAW_ZIGZAG, DRAW_ZIGZAG};
    const color Colors[] = {clrYellow, clrYellow};

    const int Size = ::ArraySize(Names);

    ::ArrayResize(this.Buffers, Size);

    for (int i = 0; i < Size; i++)
      this.Buffers[i] = new BUFFER(Types[i], EMPTY, EMPTY, Colors[i], Names[i]);

    ::IndicatorSetInteger(INDICATOR_DIGITS, ::_Digits);
  }

  virtual int Calculate( const int rates_total,
                         const int prev_calculated,
                         const datetime &time[],
                         const double &open[],
                         const double &high[],
                         const double &low[],
                         const double &close[],
                         const long &tick_volume[],
                         const long &volume[],
                         const int &spread[] )
  {
    static datetime LastTime = NULL;

    const datetime FirstTime = time[this.CorrectIndex(rates_total - prev_calculated, rates_total)];

    if (LastTime > FirstTime)
      LastTime = FirstTime;

    if (LastTime < this.StartDate)
      LastTime = this.StartDate;

    // MT4    ,    
    if (prev_calculated == 0)
      this.SetEmpty();

    LastTime = CalcIndicator(LastTime);

    return(rates_total);
  }
};