#property strict

#include "Quotes.mqh"

struct ZZ
{
  PRICE_TYPE Price;
  datetime time;

  bool operator >( const ZZ &Value ) const
  {
    return(this.Price > Value.Price);
  }

  bool operator <( const ZZ &Value ) const
  {
    return(this.Price < Value.Price);
  }

  bool operator >=( const PRICE_TYPE Value ) const
  {
    return(this.Price >= Value);
  }

  bool operator <=( const PRICE_TYPE Value ) const
  {
    return(this.Price <= Value);
  }

  PRICE_TYPE operator +( const PRICE_TYPE Value ) const
  {
    return(this.Price + Value);
  }

  PRICE_TYPE operator -( const PRICE_TYPE Value ) const
  {
    return(this.Price - Value);
  }

  string ToString( void ) const
  {
    const string Str = (string)this.time + " " + (string)this.Price;

    return(Str);
  }
};

class ZIGZAG
{
private:
  ZZ Zz[];
  int AmountFull;

  void InitZZbase( const int iMinPips = 0 )
  {
    if (this.Quotes == NULL)
      return;

//    if (!QUOTES::MyArrayResize(this.Zz, Amount, __FUNCTION__, FALSE))
//      return;
    this.AmountFull = this.Quotes.GetAmount() << 1;

    ::ArrayResize(this.Zz, this.AmountFull);

    bool FlagUP = TRUE;
    QUOTE MinMax = this.Quotes[0];
    int iAmount = 0;

    for (int i = 0; i < this.AmountFull; i++)
    {
      const int Pos = i >> 1;

      if (FlagUP)
      {
        if (MinMax - this.Quotes[Pos] >= iMinPips)
        {
          this.Zz[iAmount].Price = MinMax.bid.max;
          this.Zz[iAmount].time = MinMax.time;
          iAmount++;

          MinMax = this.Quotes[Pos];

          FlagUP = FALSE;
        }
        else if (this.Quotes[Pos] > MinMax)
          MinMax = this.Quotes[Pos];
      }
      else
      {
        if (this.Quotes[Pos] - MinMax >= iMinPips)
        {
          this.Zz[iAmount].Price = MinMax.ask.min;
          this.Zz[iAmount].time = MinMax.time;
          iAmount++;

          MinMax = this.Quotes[Pos];

          FlagUP = TRUE;
        }
        else if (this.Quotes[Pos] < MinMax)
          MinMax = this.Quotes[Pos];
      }
    }

    //  
    if (iAmount < this.AmountFull)
    {
      if (FlagUP)
        this.Zz[iAmount].Price = MinMax.bid.max;
      else
        this.Zz[iAmount].Price = MinMax.ask.min;

      this.Zz[iAmount].time = MinMax.time;

      iAmount++;
    }

    //    
//    if (!QUOTES::MyArrayResize(Output, iAmount, __FUNCTION__, TRUE, Amount - iAmount))
//      return;
    ::ArrayResize(this.Zz, iAmount, this.AmountFull - iAmount);

    return;
  }

  void InitZZbase2( const int iMinPips = 0 )
  {
    const int Amount = this.GetAmount();

    if (Amount <= 0)
      return;

    bool FlagUP = TRUE;
    ZZ MinMax = this.Zz[0];
    int TmpMinPips = MinMax - iMinPips;
    int iAmount = 0;

    for (int Pos = 0; Pos < Amount; Pos++)
    {
      if (FlagUP)
      {
        if (this.Zz[Pos] <= TmpMinPips)
        {
          this.Zz[iAmount] = MinMax;
          iAmount++;

          MinMax = this.Zz[Pos];
          TmpMinPips = MinMax + iMinPips;

          FlagUP = FALSE;
        }
        else if (this.Zz[Pos] > MinMax)
        {
          MinMax = this.Zz[Pos];
          TmpMinPips = MinMax - iMinPips;
        }
      }
      else
      {
        if (this.Zz[Pos] >= TmpMinPips)
        {
          this.Zz[iAmount] = MinMax;
          iAmount++;

          MinMax = this.Zz[Pos];
          TmpMinPips = MinMax - iMinPips;

          FlagUP = TRUE;
        }
        else if (this.Zz[Pos] < MinMax)
        {
          MinMax = this.Zz[Pos];
          TmpMinPips = MinMax + iMinPips;
        }
      }
    }

    //  
    if (iAmount < Amount)
    {
      this.Zz[iAmount] = MinMax;
      iAmount++;
    }

    //    
//    if (!QUOTES::MyArrayResize(Output, iAmount, __FUNCTION__, TRUE, Amount - iAmount))
//      return;
    ::ArrayResize(this.Zz, iAmount, this.AmountFull - iAmount);

    return;
  }

protected:
  const QUOTES* Quotes;
  int MinPips;

public:
  ZIGZAG( const QUOTES* SourceQuotes, const int iMinPips = 0 ) : Quotes(SourceQuotes), MinPips(INT_MAX)
  {
    Init(iMinPips);
  }

  void Init( const int iMinPips = 0 )
  {
//    
    if (iMinPips < this.MinPips)
      InitZZbase(iMinPips); //  ,   
    else if (iMinPips > this.MinPips)
      InitZZbase2(iMinPips);

    this.MinPips = iMinPips;

    return;
  }

  int GetMinPips( void ) const
  {
    return(this.MinPips);
  }

  int GetAmount( void ) const
  {
    return(::ArraySize(this.Zz));
  }

  string GetSymbolName( ) const
  {
    const string Str = (this.Quotes == NULL) ? "" : this.Quotes.GetSymbolName();

    return(Str);
  }

  const ZZ operator []( const int Pos ) const
  {
    return(this.Zz[Pos]);
  }
};